#!/usr/bin/env ruby

#-
# Copyright (c) 2004 AIAN
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id:$

# This script is partly substituted for the function getMedia() which is
# in the Javascript of NPR site. For more details, see the following
# source code (that file is linked by every news page).
#
#   URL: http://www.npr.org/include/javascript/jsfuncs.js
#
# getMedia() function is too complecated to substitute this script for
# it all ...

require 'getopts'
require 'open-uri'
require 'date'
require 'ymxml'

class SMILParser < YmXML::Parser
  def initialize
    @channel = []
    @state   = [:START]
  end
  attr_accessor :channel

  def create_audio_data(audio)
    data = Hash.new
    data['title'] = audio['title']
    data['src'] = audio['src']
    if data['src'] =~ /([0-9]{8})_([A-Za-z]+)_([0-9]{2})\.[^\/.]+$/
      data['date'] = $1
      data['prog'] = $2.upcase
      data['segNum'] = $3
    else
      raise "parse error"
    end
    data
  end

  def element_start(name, data)
    elem = name.split('|',3)[1]
    attr = {}
    data.each do |key, value|
      attr[key.split('|',3)[1]] = value
    end
    case elem
    when 'smil'
      @state.push(:SMIL)
    when 'meta'
      @state.push(:META)
    when 'body'
      @state.push(:BODY)
    when 'audio'
      @state.push(:AUDIO)
      begin
        @channel.push(create_audio_data(attr))
      rescue
        abort $!
      end
    end
  end

  def element_end
    @state.pop
  end

  def parse(uri)
    xml = ''
    open(uri).each do |f|
      f.each_line do |l|
        xml += l
      end
    end
    xml.gsub!(/<BR>/, '')    # for validated xml ... mmm ...)
    parser = YmXML::Parser.new(nil, '|')
    parser.parse(xml) do |etype, name, data, currentParser|
      case etype
      when :START_ELEM
        element_start(name, data)
      when :END_ELEM
        element_end
      end  
    end
    @channel
  end

end

NPR_DOMAIN = 'http://www.npr.org/'
NPR_DMG    = '/dmg/dmg.php'
NPR_PROGS  = [
  ['ATC'    , 'All Things Considered'],
  ['ME'     , 'Morning Edition'],
  ['TOTN'   , 'Talk of the Nation'],
  ['TONYCOX', 'NPR News with Tony Cox'],
  ['DAY'    , 'Day to Day'],
  ['FA'     , 'Fresh Air from WHYY'],
  ['TMF'    , 'The Motley Fool'],
  ['WESAT'  , 'Weekend Edition - Saturday'],
  ['WESUN'  , 'Weekend Edition - Sunday']
]

DATE_DEF = Date.today.strftime('%Y%m%d')
PROG_DEF = 'ME'
SEGNUM_DEF = 'ALL'

# Create USAGE string
def usage
  ret = "usage: #{$0} [-[L|P|R] [-d <date>] [-p <program>] [-s <segNo>]] [-h]\n"
  ret += "  [operations]\n"
  ret += "    -L                   list the program\n"
  ret += "    -P                   play the program\n"
  ret += "    -R                   record the program\n"
  ret += "  [options]\n"
  ret += "    -d <date>            specify the date of broadcast\n"
  ret += "          format: YYYYMMDD\n"
  ret += "          (default: today's date (e.g. #{DATE_DEF}))\n"
  ret += "    -p <program code>    choose NPR program\n"
  NPR_PROGS.each do |e|
    ret += "          #{e[0]}#{' ' * (10 - e[0].length)}#{e[1]}\n"
  end
  ret += "          (default: #{PROG_DEF})\n"
  ret += "    -s <segment number>  program segmentation number\n"
  ret += "          (default: ALL)\n"
  ret += "    -h                   show this message.\n"
end

# Create URL string used for NPR HTTP server to get SMIL file
def create_smil_uri(prgCode, showDate, segNum, mediaPreference)
  prgCode_query   = "prgCode=#{prgCode}"
  showDate_query  = "showDate=#{showDate}"
  segNum_query    = "segNum=#{segNum}"
  mediaPref_query = "mediaPref=#{mediaPreference}"
  uri = NPR_DOMAIN.chop + NPR_DMG + '?' +
    prgCode_query + '&' +
    showDate_query + '&' +
    segNum_query + '&' +
    mediaPref_query
end

# Print an available streaming LIST
def list_npr(channel)
  if channel.size == 0
    abort "no entry found"
  end
  puts ">>>> [LIST]\n"
  no = 1
  channel.each do |audio|
    puts "(#{audio['segNum']})  #{audio['title']}"
  end
end

# Play streaming
# It requires MPlayer with RealMedia plug-in
def play_npr(channel)
  if channel.size == 0
    abort "no entry found"
  end
  channel.each do |audio|
    puts ">>>> [PLAY] #{audio['title']}"
    system("mplayer #{audio['src']}")
  end
end

# Record streaming
# It requires MPlayer with RealMedia plug-in & gogo-no-coda
def record_npr(channel)
  if channel.size == 0
    abort "no entry found"
  end
  channel.each do |audio|
    puts ">>>> [REC] #{audio['title']}"
    if audio['src'] =~ /\/([^\/.]+)\.[^\/.]+$/
      basename = $1
    end
    system("mplayer -dumpstream -dumpfile #{basename}.rm #{audio['src']}")
    system("mplayer #{basename}.rm -ao pcm -aofile #{basename}.wav")
    system("gogo #{basename}.wav")
  end
end

# Main routine
# It checks for the arguments, and branch as they are.
def main
  argc = 0
  date = ""
  prog = ""
  segnum = ""

  argc = ARGV.size
  unless getopts('hLPR', "d:#{DATE_DEF}", "p:#{PROG_DEF}", "s:#{SEGNUM_DEF}")
    abort usage
  end
  # checking for the options
  begin
    # argc
    raise if argc < 1
    # program option
    raise unless NPR_PROGS.assoc($OPT_p)
    prog = $OPT_p
    # segNum option
    raise unless $OPT_s =~ /[0-9]+|ALL|all/
    segnum = $OPT_s.sub(/ALL|all/, '')
    # date (checking through the strftime() exception)
    date = Date.parse($OPT_d).strftime('%d-%b-%Y')
    # help and any process options
    raise if $OPT_h || (! $OPT_L && $OPT_P && $OPT_R)
  rescue
    abort usage
  end

  uri = create_smil_uri(prog, date, segnum, 'RM')
  channel = SMILParser.new.parse(uri)
  # processing priority is -L > -P > -R. they are exclusive mutually.
  if $OPT_L
    list_npr(channel)
  elsif $OPT_P
    play_npr(channel)
  elsif $OPT_R
    record_npr(channel)
  end

end

main
